@@ -12,19 +12,19 @@
 #
 
 import collections
-import logging
 
 from cliff import command
 from cliff import lister
 import json
-import yaml
 
 import paunch
 
+from paunch import utils
+
 
 class Apply(command.Command):
 
-    log = logging.getLogger(__name__)
+    log = None
 
     def get_parser(self, prog_name):
         parser = super(Apply, self).get_parser(prog_name)
@@ -32,15 +32,17 @@ class Apply(command.Command):
             '--file',
             metavar='<file>',
             required=True,
-            help=('YAML or JSON file containing configuration data'),
+            help=('YAML or JSON file or directory containing configuration '
+                  'data'),
         )
         parser.add_argument(
             '--label',
             metavar='<label=value>',
             dest='labels',
             default=[],
+            action='append',
             help=('Extra labels to apply to containers in this config, in the '
-                  'form label=value.'),
+                  'form --label label=value --label label2=value2.'),
         )
         parser.add_argument(
             '--managed-by',
@@ -56,23 +58,58 @@ class Apply(command.Command):
             required=True,
             help=('ID to assign to containers'),
         )
+        parser.add_argument(
+            '--default-runtime',
+            dest='default_runtime',
+            default='podman',
+            choices=['docker', 'podman'],
+            help=('Default runtime for containers. Can be docker or podman.'),
+        )
+        parser.add_argument(
+            '--container-log-path',
+            dest='cont_log_path',
+            default=None,
+            help=('Absolute directory path for container log. Works only for '
+                  'podman container engine')
+        )
+        parser.add_argument(
+            '--healthcheck-disabled',
+            dest='healthcheck_disabled',
+            action='store_true',
+            default=False,
+            help=('Whether or not we disable the containers healthchecks')
+        )
+        parser.add_argument(
+            '--cleanup',
+            dest='cleanup',
+            action='store_true',
+            default=True,
+            help=('Whether or not we delete containers missing in the config')
+        )
         return parser
 
     def take_action(self, parsed_args):
-
+        (self.log, log_file, log_level) = \
+            utils.common.configure_logging_from_args(__name__, self.app_args)
         labels = collections.OrderedDict()
         for l in parsed_args.labels:
             k, v = l.split(('='), 1)
             labels[k] = v
 
-        with open(parsed_args.file, 'r') as f:
-            config = yaml.safe_load(f)
+        config_path = parsed_args.file
+        config = utils.common.load_config(config_path)
 
         stdout, stderr, rc = paunch.apply(
             parsed_args.config_id,
             config,
-            managed_by='paunch',
-            labels=labels
+            managed_by=parsed_args.managed_by,
+            labels=labels,
+            cont_cmd=parsed_args.default_runtime,
+            log_level=log_level,
+            log_file=log_file,
+            cont_log_path=parsed_args.cont_log_path,
+            healthcheck_disabled=parsed_args.healthcheck_disabled,
+            cleanup=parsed_args.cleanup
         )
 
         return rc
@@ -80,7 +117,7 @@ class Apply(command.Command):
 
 class Cleanup(command.Command):
 
-    log = logging.getLogger(__name__)
+    log = None
 
     def get_parser(self, prog_name):
         parser = super(Cleanup, self).get_parser(prog_name)
@@ -98,18 +135,30 @@ class Cleanup(command.Command):
             default='paunch',
             help=('Override the name of the tool managing the containers'),
         )
+        parser.add_argument(
+            '--default-runtime',
+            dest='default_runtime',
+            default='podman',
+            choices=['docker', 'podman'],
+            help=('Default runtime for containers. Can be docker or podman.'),
+        )
         return parser
 
     def take_action(self, parsed_args):
+        (self.log, log_file, log_level) = \
+            utils.common.configure_logging_from_args(__name__, self.app_args)
         paunch.cleanup(
             parsed_args.config_id,
-            managed_by=parsed_args.managed_by
+            managed_by=parsed_args.managed_by,
+            cont_cmd=parsed_args.default_runtime,
+            log_level=log_level,
+            log_file=log_file
         )
 
 
 class Delete(command.Command):
 
-    log = logging.getLogger(__name__)
+    log = None
 
     def get_parser(self, prog_name):
         parser = super(Delete, self).get_parser(prog_name)
@@ -126,15 +175,30 @@ class Delete(command.Command):
             default='paunch',
             help=('Override the name of the tool managing the containers'),
         )
+        parser.add_argument(
+            '--default-runtime',
+            dest='default_runtime',
+            default='podman',
+            choices=['docker', 'podman'],
+            help=('Default runtime for containers. Can be docker or podman.'),
+        )
         return parser
 
     def take_action(self, parsed_args):
-        paunch.delete(parsed_args.config_id, parsed_args.managed_by)
+        (self.log, log_file, log_level) = \
+            utils.common.configure_logging_from_args(__name__, self.app_args)
+        paunch.delete(
+            parsed_args.config_id,
+            parsed_args.managed_by,
+            cont_cmd=parsed_args.default_runtime,
+            log_level=log_level,
+            log_file=log_file
+        )
 
 
 class Debug(command.Command):
 
-    log = logging.getLogger(__name__)
+    log = None
 
     def get_parser(self, prog_name):
         parser = super(Debug, self).get_parser(prog_name)
@@ -142,15 +206,17 @@ class Debug(command.Command):
             '--file',
             metavar='<file>',
             required=True,
-            help=('YAML or JSON file containing configuration data')
+            help=('YAML or JSON file or directory containing configuration '
+                  'data'),
         )
         parser.add_argument(
             '--label',
             metavar='<label=value>',
             dest='labels',
             default=[],
+            action='append',
             help=('Extra labels to apply to containers in this config, in the '
-                  'form label=value.')
+                  'form --label label=value --label label2=value2.'),
         )
         parser.add_argument(
             '--managed-by',
@@ -212,19 +278,27 @@ class Debug(command.Command):
             default='debug',
             help=('ID to assign to containers')
         )
+        parser.add_argument(
+            '--default-runtime',
+            dest='default_runtime',
+            default='podman',
+            choices=['docker', 'podman'],
+            help=('Default runtime for containers. Can be docker or podman.'),
+        )
         return parser
 
     def take_action(self, parsed_args):
-
+        (self.log, log_file, log_level) = \
+            utils.common.configure_logging_from_args(__name__, self.app_args)
         labels = collections.OrderedDict()
         for l in parsed_args.labels:
             k, v = l.split(('='), 1)
             labels[k] = v
 
-        with open(parsed_args.file, 'r') as f:
-            config = yaml.safe_load(f)
-
         container_name = parsed_args.container_name
+        config_path = parsed_args.file
+        config = utils.common.load_config(config_path, container_name)
+
         cconfig = {}
         cconfig[container_name] = config[container_name]
 
@@ -256,13 +330,16 @@ class Debug(command.Command):
             parsed_args.action,
             cconfig,
             parsed_args.managed_by,
-            labels=labels
+            labels=labels,
+            cont_cmd=parsed_args.default_runtime,
+            log_level=log_level,
+            log_file=log_file
         )
 
 
 class List(lister.Lister):
 
-    log = logging.getLogger(__name__)
+    log = None
 
     def get_parser(self, prog_name):
         parser = super(List, self).get_parser(prog_name)
@@ -273,10 +350,24 @@ class List(lister.Lister):
             default='paunch',
             help=('Override the name of the tool managing the containers'),
         )
+        parser.add_argument(
+            '--default-runtime',
+            dest='default_runtime',
+            default='podman',
+            choices=['docker', 'podman'],
+            help=('Default runtime for containers. Can be docker or podman.'),
+        )
         return parser
 
     def take_action(self, parsed_args):
-        configs = paunch.list(parsed_args.managed_by)
+        (self.log, log_file, log_level) = \
+            utils.common.configure_logging_from_args(__name__, self.app_args)
+        configs = paunch.list(
+            parsed_args.managed_by,
+            cont_cmd=parsed_args.default_runtime,
+            log_level=log_level,
+            log_file=log_file
+        )
         columns = [
             'config',
             'container',
@@ -287,7 +378,11 @@ class List(lister.Lister):
         data = []
         for k, v in configs.items():
             for i in v:
-                name = i.get('Name', '/')[1:]  # strip the leading slash
+                # docker has a leading slash in the name, strip it
+                if parsed_args.default_runtime == 'docker':
+                    name = i.get('Name', '/')[1:]
+                else:
+                    name = i.get('Name', '')
                 cmd = ' '.join(i.get('Config', {}).get('Cmd', []))
                 image = i.get('Config', {}).get('Image')
                 status = i.get('State', {}).get('Status')
